from rdflib import Graph, Namespace, URIRef, Literal
from rdflib.namespace import DCTERMS, FOAF, RDF, XSD  # ADMS removido daqui
import uuid


# Defina manualmente os namespaces ausentes
ADMS = Namespace("http://www.w3.org/ns/adms#")
DCAT = Namespace("http://www.w3.org/ns/dcat#")
VCARD = Namespace("http://www.w3.org/2006/vcard/ns#")


class RDFGenerator:
    @staticmethod
    def generate(form_data):
        g = Graph()
        g = RDFGenerator._bind_namespaces(g)
        dataset_id = RDFGenerator._create_dataset(g, form_data)
        RDFGenerator._add_creator(g, dataset_id, form_data)
        RDFGenerator._add_contact_point(g, dataset_id, form_data)
        RDFGenerator._add_keywords_and_themes(g, dataset_id, form_data)
        return g.serialize(format='pretty-xml')

    @staticmethod
    def _bind_namespaces(graph):
        namespaces = {
            "dcterms": DCTERMS,
            "dcat": DCAT,
            "foaf": FOAF,
            "vcard": VCARD,
            "adms": ADMS
        }
        for prefix, namespace in namespaces.items():
            graph.bind(prefix, namespace)
        return graph

    @staticmethod
    def _create_dataset(graph, data):
        dataset_id = URIRef(f"https://dados.gov.br/dados/conjuntos-dados/{uuid.uuid4()}")
        graph.add((dataset_id, RDF.type, DCAT.Dataset))
        properties = [
            (DCTERMS.title, data['title']),
            (DCTERMS.description, data['description']),
            (DCTERMS.publisher, data['publisher']),
            (DCTERMS.license, URIRef(data['license'])),
            (DCTERMS.accrualPeriodicity, data['accrualPeriodicity']),
            (DCTERMS.accessRights, data['accessRights']),
            (ADMS.version, data.get('version', '1.0'))
        ]
        for pred, value in properties:
            graph.add((dataset_id, pred, Literal(value)))
        return dataset_id

    @staticmethod
    def _add_creator(graph, dataset_id, data):
        creator_id = URIRef(f"https://dados.gov.br/dados/organizacoes/{uuid.uuid4()}")
        graph.add((dataset_id, DCTERMS.creator, creator_id))
        graph.add((creator_id, RDF.type, FOAF.Organization))
        graph.add((creator_id, FOAF.name, Literal(data['creator'])))

    @staticmethod
    def _add_contact_point(graph, dataset_id, data):
        contact_id = URIRef(f"mailto:{data['contactPoint']}")
        graph.add((dataset_id, DCAT.contactPoint, contact_id))
        graph.add((contact_id, RDF.type, VCARD.Organization))
        graph.add((contact_id, VCARD.hasEmail, contact_id))

    @staticmethod
    def _add_keywords_and_themes(graph, dataset_id, data):
        for field, predicate in [('keywords', DCAT.keyword), ('theme', DCAT.theme)]:
            if data.get(field):
                for item in data[field].split(','):
                    graph.add((dataset_id, predicate, Literal(item.strip())))