from flask import Flask, render_template, jsonify, request
from dotenv import load_dotenv
from flasgger import Swagger
from app.controllers.dcat_controller import DcatController
from google.generativeai import configure, GenerativeModel
from google.api_core import exceptions as google_exceptions
import os
import json
import google.generativeai as genai


app = Flask(__name__)
# Configuração do Swagger
SWAGGER_CONFIG = {
    "headers": [],
    "specs": [
        {
            "endpoint": "apispec",
            "route": "/api/docs/apispec.json",
            "rule_filter": lambda rule: True,
            "model_filter": lambda tag: True,
        }
    ],
    "static_url_path": "/flasgger_static",
    "swagger_ui": True,
    "specs_route": "/api/docs/",
    "title": "API DCAT-BR",
}
swagger = Swagger(app, config=SWAGGER_CONFIG)  # Configuração explícita

@app.route('/')
def index():
    return render_template('index.html', active_page='index')
@app.route('/catalogacao')
def catalogacao():
    return render_template('catalogador.html', active_page='catalogacao')

@app.route('/validacao')
def validacao():
    return render_template('validacao.html', active_page='validacao')

@app.route('/generate_rdf', methods=['POST'])
def generate_rdf():
    """Endpoint para geração de RDF
    ---
    tags:
      - DCAT
    consumes:
      - application/x-www-form-urlencoded
    parameters:
      - name: title
        in: formData
        type: string
        required: true
      - name: description
        in: formData
        type: string
        required: true
      - name: publisher
        in: formData
        type: string
        required: true
      - name: contactPoint
        in: formData
        type: string
        format: email
        required: true
      - name: license
        in: formData
        type: string
        format: uri
        required: true
      - name: accrualPeriodicity
        in: formData
        type: string
        required: true
      - name: accessRights
        in: formData
        type: string
        required: true
      - name: version
        in: formData
        type: string
      - name: keywords
        in: formData
        type: string
      - name: theme
        in: formData
        type: string
        required: true
      - name: creator
        in: formData
        type: string
        required: true
    responses:
      200:
        description: Arquivo RDF gerado
        content:
          application/rdf+xml:
            schema:
              type: string
              example: |
                <?xml version="1.0" encoding="UTF-8"?>
                <rdf:RDF>...</rdf:RDF>
      400:
        description: Erro de validação
    """
    return DcatController.generate_rdf()

@app.route('/import_rdf', methods=['POST'])
def import_rdf():
    """Endpoint para importação de RDF
    ---
    tags:
      - DCAT
    consumes:
      - multipart/form-data
    parameters:
      - name: rdfFile
        in: formData
        type: file
        required: true
    responses:
      200:
        description: Dados extraídos do RDF
        schema:
          type: object
          properties:
            title: {type: string}
            description: {type: string}
            #... todos os outros campos
      400:
        description: Erro na importação
    """
    return DcatController.import_rdf()


@app.route('/validate_rdf', methods=['POST'])
def validate_rdf():
    """Validação de arquivo RDF DCAT-BR
    ---
    tags:
      - Validação
    consumes:
      - multipart/form-data
    parameters:
      - name: rdfFile
        in: formData
        type: file
        required: true
    responses:
      200:
        description: Resultado da validação
        schema:
          type: object
          properties:
            valid:
              type: boolean
            errors:
              type: array
              items: string
    """
    return DcatController.validate_rdf()

load_dotenv()  # Carregar variáveis do .env

@app.route('/generate-with-ai', methods=['POST'])
def generate_with_ai():
    try:   

        # Verificação básica dos dados
        data = request.get_json()
        if not data or 'description' not in data or 'orgao' not in data:
            return jsonify({"error": "Dados incompletos"}), 400   
        
        # Configuração da API
        genai.configure(
          api_key=os.getenv("GEMINI_API_KEY"),
          transport='rest'    
        )
        
        #for model in genai.list_models():
        #  print(model.name)

        # Criação do modelo       
        #model = genai.GenerativeModel('gemini-1.5-flash')
        #model = genai.GenerativeModel('gemini-2.0-pro-exp')
        model = genai.GenerativeModel('gemini-2.0-flash')


        #"publisher": {"type": "string", "pattern": "Ministério|Instituto|Agência"},

        template = {
            "title": {"type": "string", "max_length": 100},
            "description": {"type": "string", "min_length": 250},
            "accrualPeriodicity": {"enum": ["Diária", "Semanal", "Mensal", "Anual"]},
            "keywords": {"type": "array", "items": {"type": "string"}},
            "contactPoint": {"type": "string", "max_length": 20},
            "publisher": {"type": "string", "max_length": 20},
            "version": {"type": "string", "max_length": 10},
            "accessRights": {"type": "uri", "format": "url"},
            "theme": {"type": "array", "items": {"type": "string"}},
            "license": {"type": "uri", "format": "url"},
            "creator": {"type": "string", "max_length": 50}            
        }
        
        # Prompt estruturado
        prompt = f"""
                **Instruções DCAT-BR:**
                Gere metadados em JSON para os seguintes inputs:
                - Descrição: {data['description'][:1000]}
                - Órgão: {data['orgao']}
                
                Regras:
                1. Gerar JSON estritamente válido;
                2. Seguir template: {json.dumps(template)};
                3. Campos desconhecidos deve ser preenchido como "string" vazia;
                4. Formatar segundo Manual DCAT-BR;
                5. Validar tipos de dados;
                6. Usar vocabulários controlados;
                7. Seguir os padrão DCAT Versão 3 da W3C;
                8. Fazer uso de URI válidas;
                10. Para o campo "accessRights" usar URL da lei vigente publicada no "Diário Oficial" ou arcabouço legal relacionado a criação do conjunto de dados que está sendo catalogado.
                Resposta APENAS em JSON:
                """       

        # Chamada à API
        response = model.generate_content(
            prompt,
            generation_config={
                "temperature": 0.1,
                "top_p": 0.95,
                "top_k": 40,
                "max_output_tokens": 2000
            },
            safety_settings={
                "HARASSMENT": "BLOCK_NONE",
                "HATE": "BLOCK_NONE",
                "SEXUAL": "BLOCK_NONE",
                "DANGEROUS": "BLOCK_NONE"
            }
        )        
        # Processamento da resposta
        try:
            cleaned_response = response.text.replace('```json', '').replace('```', '').strip()
            result = json.loads(cleaned_response)
            #print(result)
            return jsonify(result)
            
        except json.JSONDecodeError:
            return jsonify({
                "error": "Resposta em formato inválido",
                "raw_response": response.text
            }), 500

    except Exception as e:
        return jsonify({
            "error": "Erro interno - " + str(e),
            "details": str(e)
        }), 500



#@app.route('/static/<path:path>')
#def send_static(path):
#    return send_from_directory('static', path)

if __name__ == '__main__':
    app.run(debug=True)